module Scrubyt
  ##
  #=<tt>Fetching pages (and related functionality)</tt>
  #
  #Since lot of things are happening during (and before) 
  #the fetching of a document, I decided to move out fetching related
  #functionality to a separate class - so if you are looking for anything
  #which is loading a document (even by submitting a form or clicking a link)
  #and related things like setting a proxy etc. you should find it here.
  class FetchAction
    def initialize
      @@current_doc_url = nil    
      @@current_doc_protocol = nil
      @@base_dir = nil
      @@host_name = nil
      @@agent = WWW::Mechanize.new        
    end
    
    ##
    #Action to fetch a document (either a file or a http address)
    # 
    #*parameters*
    #
    #_doc_url_ - the url or file name to fetch
    def self.fetch(doc_url, proxy=nil, mechanize_doc=nil)
      parse_and_set_proxy(proxy) if proxy      
      if (mechanize_doc == nil)
        @@current_doc_url = doc_url
        @@current_doc_protocol = determine_protocol
        handle_relative_path(doc_url)
        handle_relative_url(doc_url)
                
        puts "[ACTION] fetching document: #{@@current_doc_url}"
        if @@current_doc_protocol != 'file'
          @@mechanize_doc = @@agent.get(@@current_doc_url) 
          store_host_name(doc_url)
        end
      else
        @@current_doc_url = doc_url
        @@mechanize_doc = mechanize_doc
        @@current_doc_protocol = determine_protocol
      end
      if @@current_doc_protocol == 'file'
        @@hpricot_doc = Hpricot(PreFilterDocument.br_to_newline(open(@@current_doc_url).read))
      else
        @@hpricot_doc = Hpricot(PreFilterDocument.br_to_newline(@@mechanize_doc.body))
      end
    end
    
    ##
    #Submit the last form; 
    def self.submit(current_form, button=nil)
      puts '[ACTION] submitting form...'
      if button == nil 
        result_page = @@agent.submit(current_form)
      else
        result_page = @@agent.submit(current_form, button)
      end
      @@current_doc_url = result_page.uri.to_s
      puts "[ACTION] fetched #{@@current_doc_url}"
      fetch(@@current_doc_url, nil, result_page)
    end
    
    ##
    #Click the link specified by the text    
    def self.click_link(link_text)
      puts "[ACTION] clicking link: #{link_text}"
      link = @@mechanize_doc.links.text(/^#{Regexp.escape(link_text)}$/)
      result_page = @@agent.click(link)
      @@current_doc_url = result_page.uri.to_s
      puts "[ACTION] fetched #{@@current_doc_url}"
      fetch(@@current_doc_url, nil, result_page)      
    end       
    
    ##
    # At any given point, the current document can be queried with this method; Typically used
    # when the navigation is over and the result document is passed to the wrapper 
    def self.get_current_doc_url
      @@current_doc_url
    end
    
    def self.get_mechanize_doc
      @@mechanize_doc
    end
    
    def self.get_hpricot_doc
      @@hpricot_doc
    end       
private    
    def self.determine_protocol
      old_protocol = @@current_doc_protocol
      new_protocol = case @@current_doc_url 
        when /^https/
          'https'
        when /^http/
          'http'
        when /^www/
          'http'
        else
          'file'
        end
      return 'http' if ((old_protocol == 'http') && new_protocol == 'file')
      return 'https' if ((old_protocol == 'https') && new_protocol == 'file')
      new_protocol
    end    
    
    def self.parse_and_set_proxy(proxy)
      proxy = proxy[:proxy]    
      if proxy.downcase == 'localhost'
        @@host = 'localhost'
        @@port = proxy.split(':').last
      else
        parts = proxy.split(':')
        @@port = parts.delete_at(-1)
        @@host = parts.join(':')
        if (@@host == nil || @@port == nil)# !@@host =~ /^http/)
          puts "Invalid proxy specification..."
          puts "neither host nor port can be nil!"
          exit
        end
      end
      puts "[ACTION] Setting proxy: host=<#{@@host}>, port=<#{@@port}>"
      @@agent.set_proxy(@@host, @@port)
    end
    
    def self.handle_relative_path(doc_url)
      if @@base_dir == nil
        @@base_dir = doc_url.scan(/.+\//)[0] if @@current_doc_protocol == 'file'
      else        
        @@current_doc_url = ((@@base_dir + doc_url) if doc_url !~ /#{@@base_dir}/)
      end
    end
        
    def self.handle_relative_url(doc_url)
      return if doc_url =~ /^http/
      if @@host_name != nil        
        if doc_url !~ /#{@@host_name}/
          @@current_doc_url = (@@host_name + doc_url)
          #remove duplicate parts, like /blogs/en/blogs/en
          @@current_doc_url = @@current_doc_url.split('/').uniq.reject{|x| x == ""}.join('/')
          @@current_doc_url.sub!('http:/', 'http://')
        end  
      end
    end
    
    def self.store_host_name(doc_url)
      @@host_name = 'http://' + @@mechanize_doc.uri.to_s.scan(/http:\/\/(.+\/)+/).flatten[0] if @@current_doc_protocol == 'http'
      @@host_name = 'https://' + @@mechanize_doc.uri.to_s.scan(/https:\/\/(.+\/)+/).flatten[0] if @@current_doc_protocol == 'https'
      @@host_name = doc_url if @@host_name == nil        
    end #end of function store_host_name
  end #end of class FetchAction
end #end of module Scrubyt
