module Scrubyt
  ##
  #=<tt>Utility class for adding constraints</tt>
  #
  #Originally methods of Pattern - but since Pattern was already too heavy (and after 
  #all, adding a constraint (logically) does not belong to Pattern anyway) it was moved
  #to this utility class. In pattern everything that begins with ensure_ 
  #is automatically dispatched here.
  #
  #I will not document the functions since these are just forwarders; See the 'real'
  #functions with their documentation in Scrubyt::Constraint.rb
  class ConstraintAdder
  
    def self.ensure_presence_of_pattern(pattern, ancestor_node_name)    
      pattern.filters[0].ensure_presence_of_pattern(ancestor_node_name)
      pattern #To make chaining possible
    end
    
    def self.ensure_presence_of_ancestor_node(pattern, ancestor_node_name, attributes=[])
       pattern.filters[0].ensure_presence_of_ancestor_node(ancestor_node_name, 
                                                           prepare_attributes(attributes))
       pattern #To make chaining possible
    end
    
    def self.ensure_absence_of_ancestor_node(pattern, ancestor_node_name, attributes=[])
      pattern.filters[0].ensure_absence_of_ancestor_node(ancestor_node_name, 
                                                         prepare_attributes(attributes))
      pattern #To make chaining possible
    end
    
    def self.ensure_presence_of_attribute(pattern, attribute_hash)
       pattern.filters[0].ensure_presence_of_attribute(attribute_hash)
       pattern #To make chaining possible
    end
    
    def self.ensure_absence_of_attribute(pattern, attribute_hash)
       pattern.filters[0].ensure_absence_of_attribute(attribute_hash)
       pattern #To make chaining possible
    end        
    
private
    def self.find_by_name(root_pattern, name)
      @found_pattern = nil
      find_by_name_recursive(root_pattern, name)
      if (@found_pattern == nil)
        #$Logger.error("Fatal: No pattern named #{name} exists!")
	puts "Fatal: No pattern named #{name} exists!"
      end
      @found_pattern      
    end
    
    def self.find_by_name_recursive(pattern, name)
      if pattern.name == name
        @found_pattern = pattern
      else
        pattern.children.each {|child| find_by_name_recursive(child, name)}
      end
    end
    
    def self.prepare_attributes(attributes)
      attribute_pairs = []
      attributes.each do |key, value|
        if (value.instance_of? Array)
          value.each {|val| attribute_pairs << [key,val]}          
        else
          attribute_pairs << [key, value]
        end
      end
      return attribute_pairs
    end #end of method prepare_attributes      
  end #end of class ConstraintAddere
end #end of module Scrubyt
