require 'logger'
require 'open-uri'
require 'rubygems'
require 'mechanize'
require 'hpricot'

module Scrubyt
##
#=<tt>Driving the whole extraction process</tt>
#
#Extractor is a performer class - it gets an extractor definition and carries
#out the actions and evaluates the wrappers sequentially. 
#
#Originally also the navigation actions were here, but since the class got too
#big, they were factored out to an own class, NavigationAction.
  class Extractor 
    #The definition of the extractor is passed through this method
    def self.define(mode=nil, &extractor_definition)
      @@mode = mode
      mode_name = (mode == :production ? 'Production' : 'Learning')
      puts "[MODE] #{mode_name}"
      NavigationActions.new
      @@evaluation_context = EvaluationContext.new
      #Hack up an artificial root pattern (i.e. do not return the pattern which 
      #is the root one in the user's definition, but rather the real (invisible)
      #root pattern
      evaluated_extractor = (class_eval(&extractor_definition))
      if evaluated_extractor == nil
        puts "No extractor defined, exiting..."
        exit
      end
      root_pattern = evaluated_extractor.parent
      #Recursively match data based on examples
      @@evaluation_context.setup_examples
      #Once all is set up, evaluate the extractor from the root pattern!
      evaluate_extractor(root_pattern)
      #Apply all postprocess steps
      PostProcessor.apply_post_processing(root_pattern)
      #Return the root pattern
      puts "Extraction finished succesfully!"
      root_pattern
    end

  #build the current wrapper    
  def self.method_missing(method_name, *args, &block)
    if NavigationActions::KEYWORDS.include? method_name.to_s
      NavigationActions.send(method_name, *args)
      return
    end
    pattern = Scrubyt::Pattern.new(method_name.to_s, *args)
    pattern.evaluation_context = @@evaluation_context
    if @parent == nil
      if method_name.to_s == 'next_page'
        @@evaluation_context.next_page = args[0]
        @@evaluation_context.limit = 
          args[1][:limit] if args.size > 1
        return @@last_pattern
      else
        #Create a root pattern
        root_pattern = Scrubyt::Pattern.new('root', :type => :root) 
        root_pattern.evaluation_context = @@evaluation_context
        @@evaluation_context.root_pattern = root_pattern
        @@evaluation_context.extractor = self
        #add the currently active document to the root pattern 
        @@evaluation_context.attach_current_document
        @@evaluation_context.root_pattern.add_child_pattern(pattern)        
        @@evaluation_context.block_count = 0
      end
    else
      @parent.add_child_pattern(pattern) if @parent != nil
    end
    if block_given?
      @@evaluation_context.block_count = @@evaluation_context.block_count + 1
      @stack ||=[]
      @parent = pattern
      @stack.push @parent
      class_eval(&block)
      @stack.pop
      @parent = @stack.last
    end    
    @@last_pattern = pattern
  end

  #Used in lord of the hacks vol 1. Check out export.rb if you are still interested
  #(You should not be :)    
  def self.get_block_count
    @@root_pattern.block_count
  end
  
  def self.get_hpricot_doc
    NavigationActions.get_hpricot_doc
  end
  
  def self.get_mode
    @@mode
  end  
private
    def self.evaluate_extractor(root_pattern)  
      if @@evaluation_context.next_page
        current_page_count = 1
        loop do
          really_evaluate_extractor(root_pattern)
          break if (@@evaluation_context.limit == current_page_count || @@evaluation_context.crawl_to_new_page == nil)
          current_page_count += 1 if @@evaluation_context.limit != nil
        end
      else 
        really_evaluate_extractor(root_pattern)
      end
    end  
    
    def self.really_evaluate_extractor(pattern)
      pattern.evaluate
      pattern.children.each { |child| really_evaluate_extractor child } 
    end #end of method evaluate_wrapper    
  end #end of class Extractor
end #end of module Scrubyt