require 'rexml/document'
require 'rexml/xpath'

module Scrubyt
  ##
  #=<tt>Dumping the result in various formats and providing statistics on the results</tt>
  class ResultDumper
    ##
    #Output the results as XML
    def self.to_xml(pattern)
      doc = REXML::Document.new
      root = REXML::Element.new('root')
      doc.add_element(root)
      all_extracted_docs = pattern.last_result
      all_extracted_docs.each do |lr| 
        pattern.last_result = lr
        to_xml_recursive(pattern, root) 
      end
      remove_empty_leaves(doc)
      @@last_doc = doc
    end
    
    def self.remove_empty_leaves(node)
      node.remove if  node.elements.empty? && node.text == nil
      node.elements.each {|child| remove_empty_leaves child }
    end
    
    ##
    #Output the text of the pattern; If this pattern is a tree, collect the text from its
    #result instance node; otherwise rely on the last_result
    def self.to_text(pattern)
       last_result = pattern.last_result
       result = ""
       if pattern.type == Scrubyt::Pattern::PATTERN_TYPE_TREE 
         last_result.traverse_text { |t| result += t.to_s }
       else
         result = last_result
       end
       result
    end
    
    ##
    #Print some simple statistics on the extracted results, like the count of extracted 
    #instances by each pattern    
    def self.print_statistics(pattern)
      puts "\n" * 2
      print_statistics_recursive(pattern,0)
      puts
    end

private
    def self.to_xml_recursive(pattern, element)
      pattern.children.each do |child|        
        childresults = child.result.lookup(child.parent.last_result)
        #Output text for leaf nodes only; Maybe add possibility to customize this later
        if (childresults == nil)
          res = ""
          child.parent.last_result.traverse_text { |t| res += t.to_s }
          if (child.parent.size == 0)
            element.text = (res.gsub('&nbsp;'){' '}).strip unless element.parent.is_a? REXML::Document
          end
          next
        end
        generate_children(child, childresults, element)
      end
    end
    
    def self.generate_children(child, childresults, element)
        childresults.size.times do |num|
          child.last_result = childresults[num]          
          res = ""
          if child.last_result.instance_of? String
            res = child.last_result
          else
            child.last_result.traverse_text { |t| res += t.to_s } if child.last_result != nil
          end
          child_node = REXML::Element.new(child.name)          
          child_node.text = (res.gsub('&nbsp;'){' '}).strip if (child.children.size == 0)
          element.add_element(child_node)
          to_xml_recursive(child, child_node)
        end    
    end
    
    def self.print_statistics_recursive(pattern, depth)
      if pattern.name != 'root'
        count = REXML::XPath.match(@@last_doc, "//#{pattern.name}").size 
        puts((' ' * "#{depth}".to_i) +  "#{pattern.name} extracted #{count} instances.")
      end
      
      pattern.children.each do |child|
        print_statistics_recursive(child, depth + 4)
      end      
    end#end of method print_statistics_recursive
  end #end of class ResultDumper      
end #end of module Scrubyt
