#require File.join(File.dirname(__FILE__), '../..', 'lib', 'filter')
require 'scrubyt'
require 'test/unit'

class FilterTest < Test::Unit::TestCase
  def test_determine_example_type
    #Test children example
    assert_equal(Scrubyt::Filter.determine_example_type(nil), 
                 Scrubyt::Filter::EXAMPLE_TYPE_CHILDREN)                 
    #Test image example
    assert_equal(Scrubyt::Filter.determine_example_type('scrubyt.png'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_IMAGE)
    assert_equal(Scrubyt::Filter.determine_example_type('scrubyt.gif'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_IMAGE)
    assert_equal(Scrubyt::Filter.determine_example_type('scrubyt.jpg'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_IMAGE)
    assert_equal(Scrubyt::Filter.determine_example_type('scrubyt.jpeg'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_IMAGE)                 
    assert_not_equal(Scrubyt::Filter.determine_example_type('scrubyt.zip'), 
                     Scrubyt::Filter::EXAMPLE_TYPE_IMAGE)
    assert_not_equal(Scrubyt::Filter.determine_example_type('scrubyt.pif'), 
                     Scrubyt::Filter::EXAMPLE_TYPE_IMAGE)
    #Test XPaths
    assert_equal(Scrubyt::Filter.determine_example_type('/p/img'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)                 
    assert_equal(Scrubyt::Filter.determine_example_type('/p/h3'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)                 
    assert_equal(Scrubyt::Filter.determine_example_type('/p/h3/a/h2'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('/h2'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('/h1/h3'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)                                 
    assert_equal(Scrubyt::Filter.determine_example_type('/p'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('//p'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('/p//img'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('//p//img'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('/p[0]/img'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('/p[0]'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('//p[1]'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('/p[1]//img[2]'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('//p[1]//img'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('/table/tr/td//span/b'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_equal(Scrubyt::Filter.determine_example_type('/table[0]//tr/td[1]/span[2]/b'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_not_equal(Scrubyt::Filter.determine_example_type('table[0]//tr/td[1]/span[2]/b'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)                 
    assert_not_equal(Scrubyt::Filter.determine_example_type('/table[a]//tr/td[1]/span[2]/b'),
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)                 
    assert_not_equal(Scrubyt::Filter.determine_example_type('/tab2le[a]//tr/td[1]/span[2]/b'),
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    assert_not_equal(Scrubyt::Filter.determine_example_type('/table[a]///tr/td[1]/span[2]/b'),
                 Scrubyt::Filter::EXAMPLE_TYPE_XPATH)
    #Test string example
    assert_equal(Scrubyt::Filter.determine_example_type('Hello, world!'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_STRING)
    assert_equal(Scrubyt::Filter.determine_example_type('$1022'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_STRING)
    assert_equal(Scrubyt::Filter.determine_example_type('CANON'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_STRING)
    assert_equal(Scrubyt::Filter.determine_example_type('This is a string'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_STRING)
    assert_equal(Scrubyt::Filter.determine_example_type('45'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_STRING)
    assert_equal(Scrubyt::Filter.determine_example_type('td'), 
                 Scrubyt::Filter::EXAMPLE_TYPE_STRING)

  end
end
